@@ -26,7 +26,7 @@ from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 
 app_name = "ReText"
-app_version = "2.1.5"
+app_version = "2.1.6"
 
 settings = QSettings('ReText project', 'ReText')
 
@@ -40,7 +40,11 @@ else:
 		exts = []
 		for ext in settings.value('mdExtensions').toStringList():
 			exts.append(str(ext))
-		md = markdown.Markdown(exts)
+		try:
+			md = markdown.Markdown(exts)
+		except ValueError:
+			print('Warning: failed to load extensions!')
+			md = markdown.Markdown()
 	else:
 		md = markdown.Markdown()
 
@@ -118,7 +122,7 @@ class ReTextHighlighter(QSyntaxHighlighter):
 					self.setFormat(index, length, charFormat)
 				index = expression.indexIn(text, index + length)
 		charFormat = QTextCharFormat()
-		patterns = ('<[^<>]*>', '&[^; ]*;', '"[^"<]*"(?=[^<]*>)', '<!--[^-->]*-->')
+		patterns = ('<[^<>@]*>', '&[^; ]*;', '"[^"<]*"(?=[^<]*>)', '<!--[^-->]*-->')
 		foregrounds = (Qt.darkMagenta, Qt.darkCyan, Qt.darkYellow, Qt.gray)
 		for i in range(len(patterns)):
 			expression = QRegExp(patterns[i])
@@ -283,7 +287,7 @@ class ReTextWindow(QMainWindow):
 		self.actionQuit = QAction(self.actIcon('application-exit'), self.tr('Quit'), self)
 		self.actionQuit.setShortcut(QKeySequence.Quit)
 		self.actionQuit.setMenuRole(QAction.QuitRole)
-		self.connect(self.actionQuit, SIGNAL('triggered()'), qApp, SLOT('quit()'))
+		self.connect(self.actionQuit, SIGNAL('triggered()'), self.close)
 		self.actionUndo = QAction(self.actIcon('edit-undo'), self.tr('Undo'), self)
 		self.actionUndo.setShortcut(QKeySequence.Undo)
 		self.actionRedo = QAction(self.actIcon('edit-redo'), self.tr('Redo'), self)
@@ -768,36 +772,37 @@ class ReTextWindow(QMainWindow):
 			self.openFileWrapper(fileName)
 	
 	def openFileWrapper(self, fileName):
-		if fileName:
-			exists = False
-			for i in range(self.tabWidget.count()):
-				if self.fileNames[i] == fileName:
-					exists = True
-					ex = i
-			if exists:
-				self.tabWidget.setCurrentIndex(ex)
-			else:
-				if self.fileNames[self.ind] or self.editBoxes[self.ind].toPlainText() \
-				or self.editBoxes[self.ind].document().isModified():
-					self.tabWidget.addTab(self.createTab(""), "")
-					self.ind = self.tabWidget.count()-1
-					self.tabWidget.setCurrentIndex(self.ind)
-				self.fileNames[self.ind] = fileName
-				self.openFileMain()
+		if not fileName:
+			return
+		fileName = QFileInfo(fileName).canonicalFilePath()
+		exists = False
+		for i in range(self.tabWidget.count()):
+			if self.fileNames[i] == fileName:
+				exists = True
+				ex = i
+		if exists:
+			self.tabWidget.setCurrentIndex(ex)
+		elif QFile.exists(fileName):
+			if self.fileNames[self.ind] or self.editBoxes[self.ind].toPlainText() \
+			or self.editBoxes[self.ind].document().isModified():
+				self.tabWidget.addTab(self.createTab(""), "")
+				self.ind = self.tabWidget.count()-1
+				self.tabWidget.setCurrentIndex(self.ind)
+			self.fileNames[self.ind] = fileName
+			self.openFileMain()
 	
 	def openFileMain(self):
-		if QFile.exists(self.fileNames[self.ind]):
-			openfile = QFile(self.fileNames[self.ind])
-			openfile.open(QIODevice.ReadOnly)
-			html = QTextStream(openfile).readAll()
-			openfile.close()
-			self.editBoxes[self.ind].setPlainText(html)
-			suffix = QFileInfo(self.fileNames[self.ind]).suffix()
-			pt = not (suffix in ('re', 'md', 'markdown', 'mdown', 'mkd', 'mkdn', 'rst', 'rest', 'html', 'htm'))
-			self.actionPlainText.setChecked(pt)
-			self.enablePlainText(pt)
-			self.setCurrentFile()
-			self.setWindowModified(False)
+		openfile = QFile(self.fileNames[self.ind])
+		openfile.open(QIODevice.ReadOnly)
+		html = QTextStream(openfile).readAll()
+		openfile.close()
+		self.editBoxes[self.ind].setPlainText(html)
+		suffix = QFileInfo(self.fileNames[self.ind]).suffix()
+		pt = not (suffix in ('re', 'md', 'markdown', 'mdown', 'mkd', 'mkdn', 'rst', 'rest', 'html', 'htm'))
+		self.actionPlainText.setChecked(pt)
+		self.enablePlainText(pt)
+		self.setCurrentFile()
+		self.setWindowModified(False)
 	
 	def saveFile(self):
 		self.saveFileMain(dlg=False)
@@ -833,22 +838,22 @@ class ReTextWindow(QMainWindow):
 			if self.fileNames[self.ind] and QFileInfo(self.fileNames[self.ind]).suffix().isEmpty():
 				self.fileNames[self.ind].append(ext)
 		if self.fileNames[self.ind]:
-			self.setCurrentFile()
-		if QFileInfo(self.fileNames[self.ind]).isWritable() or not QFile.exists(self.fileNames[self.ind]):
-			if self.fileNames[self.ind]:
-				self.saveFileWrapper(self.fileNames[self.ind])
+			result = self.saveFileWrapper(self.fileNames[self.ind])
+			if result:
+				self.setCurrentFile()
 				self.editBoxes[self.ind].document().setModified(False)
 				self.setWindowModified(False)
-		else:
-			self.setWindowModified(self.isWindowModified())
-			QMessageBox.warning(self, app_name, self.tr("Cannot save to file because it is read-only!"))
+			else:
+				QMessageBox.warning(self, app_name, self.tr("Cannot save to file because it is read-only!"))
 	
 	def saveFileWrapper(self, fn):
 		savefile = QFile(fn)
-		savefile.open(QIODevice.WriteOnly)
-		savestream = QTextStream(savefile)
-		savestream << self.editBoxes[self.ind].toPlainText()
-		savefile.close()
+		result = savefile.open(QIODevice.WriteOnly)
+		if result:
+			savestream = QTextStream(savefile)
+			savestream << self.editBoxes[self.ind].toPlainText()
+			savefile.close()
+		return result
 	
 	def saveHtml(self, fileName):
 		if QFileInfo(fileName).suffix().isEmpty():
@@ -865,14 +870,14 @@ class ReTextWindow(QMainWindow):
 			html << text << "\n"
 			htmlFile.close()
 			return
-		html << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"
-		html << "<html>\n<head>\n"
-		html << "  <meta http-equiv=\"content-type\" content=\"text/html; charset=utf-8\">\n"
-		html << QString("  <meta name=\"generator\" content=\"%1 %2\">\n").arg(app_name, app_version)
-		html << "  <title>" + self.getDocumentTitle() + "</title>\n"
-		html << "</head>\n<body>\n"
+		html << '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">\n'
+		html << '<html>\n<head>\n'
+		html << '  <meta http-equiv="content-type" content="text/html; charset=utf-8">\n'
+		html << '  <meta name="generator" content="%s %s">\n' % (app_name, app_version)
+		html << '  <title>' + self.getDocumentTitle() + '</title>\n'
+		html << '</head>\n<body>\n'
 		html << text
-		html << "\n</body>\n</html>\n"
+		html << '\n</body>\n</html>\n'
 		htmlFile.close()
 	
 	def textDocument(self):
@@ -1102,14 +1107,10 @@ class ReTextWindow(QMainWindow):
 		return True
 	
 	def closeEvent(self, closeevent):
-		accept = True
 		for self.ind in range(self.tabWidget.count()):
 			if not self.maybeSave(self.ind):
-				accept = False
-		if accept:
-			closeevent.accept()
-		else:
-			closeevent.ignore()
+				return closeevent.ignore()
+		closeevent.accept()
 	
 	def viewHtml(self):
 		HtmlDlg = HtmlDialog(self)
@@ -1157,16 +1158,10 @@ class ReTextWindow(QMainWindow):
 	def getParser(self):
 		if self.fileNames[self.ind]:
 			suffix = QFileInfo(self.fileNames[self.ind]).suffix()
-			if suffix in ('md', 'markdown', 'mdown', 'mkd', 'mkdn'):
-				if use_md:
-					return PARSER_MARKDOWN
-				else:
-					return PARSER_NA
+			if suffix in ('md', 'markdown', 'mdown', 'mkd', 'mkdn', 're'):
+				return PARSER_MARKDOWN if use_md else PARSER_NA
 			elif suffix in ('rest', 'rst'):
-				if use_docutils:
-					return PARSER_DOCUTILS
-				else:
-					return PARSER_NA
+				return PARSER_DOCUTILS if use_docutils else PARSER_NA
 			elif suffix in ('html', 'htm'):
 				return PARSER_HTML
 		if not (use_docutils or use_md):
